"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@checkup/core");
const recast_1 = require("recast");
const fs = require('fs');
const ESLINT_DISABLE_REGEX = /^eslint-disable(?:-next-line|-line)*/gi;
class EslintDisableTask extends core_1.BaseTask {
    constructor() {
        super(...arguments);
        this.taskName = 'eslint-disables';
        this.taskDisplayName = 'Number of eslint-disable Usages';
        this.category = 'linting';
    }
    async run() {
        let jsPaths = this.context.paths.filterByGlob('**/*.js');
        let eslintDisables = await getEslintDisables(jsPaths, this.context.cliFlags.cwd);
        return this.toJson([core_1.buildSummaryResult('eslint-disable', eslintDisables)]);
    }
}
exports.default = EslintDisableTask;
async function getEslintDisables(filePaths, cwd) {
    let data = [];
    class ESLintDisableAccumulator {
        constructor(filePath) {
            this.filePath = filePath;
            this.data = [];
        }
        get visitors() {
            let add = (node) => {
                this.data.push({
                    filePath: core_1.normalizePath(this.filePath, cwd),
                    ruleId: 'no-eslint-disable',
                    message: 'eslint-disable is not allowed',
                    line: node.loc.start.line,
                    column: node.loc.start.column,
                });
            };
            return {
                visitComment: function (path) {
                    if (path.value.value.trim().match(ESLINT_DISABLE_REGEX)) {
                        add(path.value);
                    }
                    this.traverse(path);
                },
            };
        }
    }
    await Promise.all(filePaths.map((filePath) => {
        return fs.promises.readFile(filePath, 'utf8').then((fileContents) => {
            let accumulator = new ESLintDisableAccumulator(filePath);
            let astTraverser = new core_1.AstTraverser(fileContents, recast_1.parse, recast_1.visit, {
                parser: require('recast/parsers/babel'),
            });
            astTraverser.traverse(accumulator.visitors);
            data.push(...accumulator.data);
        });
    }));
    return data;
}
//# sourceMappingURL=eslint-disable-task.js.map